//============================================================================
// Name        : graph.cpp
// Author      : Radosaw Kdzior
// Description : Program prezentujcy tworzenie prostych grafw oraz rnice
//					pomidzy grafami skierowanymi, nieskierowanymi i dwukierunkowymi.
//============================================================================


#include <boost/graph/adjacency_list.hpp>
#include <boost/graph/graph_traits.hpp>
#include <iostream>
using namespace boost;

/*
 * Tworzenie grafu znajduje si w funkcji main() - na kocu pliku.
 */


//Szablon klasy wypisujcej graf na konsol.
template <class Graph> struct exercise_vertex {
  exercise_vertex(Graph& g_) : g(g_) { }
  typedef typename graph_traits<Graph>::vertex_descriptor Vertex;
  void operator()(const Vertex& v) const
  {
    using namespace boost;
    typename property_map<Graph, vertex_index_t>::type 
      vertex_id = get(vertex_index, g);
    std::cout << "vertex: " << get(vertex_id, v);

    /*
     * Wypisanie krawdzi wychodzcych z wierzchoka.
     */
    std::cout << "\tout-edges: ";
    typename graph_traits<Graph>::out_edge_iterator out_i, out_end;
    typename graph_traits<Graph>::edge_descriptor e;
    for (tie(out_i, out_end) = out_edges(v, g);  
         out_i != out_end; ++out_i)
    {
      e = *out_i;
      Vertex src = source(e, g), targ = target(e, g);
      std::cout << "(" << get(vertex_id, src)
                << "," << get(vertex_id, targ) << ") ";
    }

    /*
     * Wypisanie krawdzi przychodzcych do wierzchoka.
     */
    std::cout << "\tin-edges: ";
    typename graph_traits<Graph>::in_edge_iterator in_i, in_end;
    for (tie(in_i, in_end) = in_edges(v, g); in_i != in_end; ++in_i)
    {
      e = *in_i;
      Vertex src = source(e, g), targ = target(e, g);
      std::cout << "(" << get(vertex_id, src)
                << "," << get(vertex_id, targ) << ") ";
    }

    /*
     * Wypisanie ssiadw wierzchoka - innych wierzchokw do ktrych mamy
     * bezporedni dostp z tego wierzchoka (do ktrych wskazuj krawdzie
     * wychodzce).
     */
    std::cout << "\tadjacent vertices: ";
    typename graph_traits<Graph>::adjacency_iterator ai, ai_end;
    for (tie(ai,ai_end) = adjacent_vertices(v, g);  ai != ai_end; ++ai)
      std::cout << get(vertex_id, *ai) <<  " ";
    std::cout << std::endl;
  }
  
  Graph& g;
};

/*
 * Szablon podobny po poprzedniego tylko dla grafw skierowanych.
 * Grafy te nie maj wiedzy o krawdziach przychodzcych
 * nie ma metody in_edges() dla takich grafw.
 */
template <class Graph> struct exercise_vertex_d {
  exercise_vertex_d(Graph& g_) : g(g_) { }
  typedef typename graph_traits<Graph>::vertex_descriptor Vertex;
  void operator()(const Vertex& v) const
  {
    using namespace boost;
    typename property_map<Graph, vertex_index_t>::type 
      vertex_id = get(vertex_index, g);
    std::cout << "vertex: " << get(vertex_id, v);

    std::cout << "\tout-edges: ";
    typename graph_traits<Graph>::out_edge_iterator out_i, out_end;
    typename graph_traits<Graph>::edge_descriptor e;
    for (tie(out_i, out_end) = out_edges(v, g);  
         out_i != out_end; ++out_i)
    {
      e = *out_i;
      Vertex src = source(e, g), targ = target(e, g);
      std::cout << "(" << get(vertex_id, src)
                << "," << get(vertex_id, targ) << ") ";
    }

    std::cout << "\tadjacent vertices: ";
    typename graph_traits<Graph>::adjacency_iterator ai, ai_end;
    for (tie(ai,ai_end) = adjacent_vertices(v, g);  ai != ai_end; ++ai)
      std::cout << get(vertex_id, *ai) <<  " ";
    std::cout << std::endl;
  }
  
  Graph& g;
};

int main() 
{
	/*
	 * Tworzenie typu grafu. Pierwsze dwa argumenty szablonu to listy (okrelamy typ)
	 * w jakich przechowywane bd odpowiednio krawdzie i wierzchoki. Mamy do wyboru:
	 * vecS		- std::vector.
	 * listS	- std::list.
	 * slistS	- std::slist.
	 * setS		- std::set.
	 * multisetS- std::multiset.
	 * hash_setS- std::hash_set.
	 * Trzeci parametr to typ grafu, tutaj kolejno: dwukierunkowe, skierowane i nieskierowane.
	 * Istnieje inna wersja szablonu adjacency_list ktra pozwala definiowa odpowiednie w waciwoci
	 * dla grafu, wierzchoku i krawdzi.
	 */
	typedef adjacency_list<vecS, vecS, bidirectionalS> Graph_bi;
	typedef adjacency_list<vecS, vecS, directedS> Graph_d;
	typedef adjacency_list<vecS, vecS, undirectedS> Graph_und;
	
	typedef std::pair<int, int> Edge;
	
	enum { A, B, C, D, E, N };
	const int num_vertices = N;

	// Definiujemy krawdzie pomidzy wierzchokami w postaci tablicy.
	Edge edge_array[] = {
							Edge(A,B), Edge(A,D), Edge(C,A), Edge(D,C),
							Edge(C,E), Edge(B,D), Edge(D,E)
						};
	
	const int num_edges = sizeof( edge_array )/sizeof( edge_array[0] );
	
	// Definicje grafw:
	Graph_bi g_bi( num_vertices );
	Graph_d g_d( num_vertices );
	Graph_und g_und( num_vertices );

	/*
	 * Dodajemy kolejno krawdzie do grafu, przy okazji definiujemy wierzchoki.
	 * Krawdzie w tym przypadku s reprezentowane przez pary liczb cakowitych,
	 * wierzchoki za przez liczby.
	 */
	for ( int i = 0; i < num_edges; ++i )
	{
		add_edge( edge_array[i].first, edge_array[i].second, g_bi );
		
		add_edge( edge_array[i].first, edge_array[i].second, g_d );
		
		add_edge( edge_array[i].first, edge_array[i].second, g_und );
	}
	

	
	/*
	 * W tym momencie mamy zdefiniowane grafy. Rnice jakie wystpuj pomidzy grafami to:
	 * Bidirectional: osobno wyrniamy krawdzie wychodzce i przychodzce
	 * 				 (dwa zbiory krawdzi dla kadego wierzchoka = wicej potrzebnych zasobw)
	 * Directed: przechowujemy tylko informacje o krawdziach wychodzcych z wierzchokw
	 * 				(nie znamy wprost krawdzi przychodzcych)
	 * Undireced: grafy nieskierowane, krawdzie s dwukierunkowe (w pamici przy wierzchoku
	 * 				jest jeden zbir krawdzi)
	 * 
	 * Poniższy kod wypisuje zawarto grafw na konsol:
	 */
	std::cout << "BIDIRECTIONAL" << std::endl;
	std::for_each(vertices(g_bi).first, vertices(g_bi).second, exercise_vertex<Graph_bi>(g_bi));
	
	std::cout << "DIRECTED" << std::endl;
	std::for_each(vertices(g_d).first, vertices(g_d).second, exercise_vertex_d<Graph_d>(g_d) );
	
	std::cout << "UNDIRECTED" << std::endl;
	std::for_each(vertices(g_und).first, vertices(g_und).second, exercise_vertex<Graph_und>(g_und));
	return 0;
}
